package cn.newfeifan.mall.module.distri.service.shopsettlement;

import javax.validation.*;

import cn.newfeifan.mall.module.distri.controller.admin.ptsettlement.vo.TransferTemplateRespVO;
import cn.newfeifan.mall.module.distri.controller.admin.shopsettlement.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.shopsettlement.ShopSettlementDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import java.util.List;

/**
 * 订单结算信息表，记录店铺的结算信息 Service 接口
 *
 * @author 非繁人
 */
public interface ShopSettlementService {

    /**
     * 创建订单结算信息表，记录店铺的结算信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createShopSettlement(@Valid ShopSettlementSaveReqVO createReqVO);

    /**
     * 更新订单结算信息表，记录店铺的结算信息
     *
     * @param updateReqVO 更新信息
     */
    void updateShopSettlement(@Valid ShopSettlementSaveReqVO updateReqVO);

    /**
     * 删除订单结算信息表，记录店铺的结算信息
     *
     * @param id 编号
     */
    void deleteShopSettlement(Long id);

    /**
     * 获得订单结算信息表，记录店铺的结算信息
     *
     * @param id 编号
     * @return 订单结算信息表，记录店铺的结算信息
     */
    ShopSettlementDO getShopSettlement(Long id);

    /**
     * 获得订单结算信息表，记录店铺的结算信息分页
     *
     * @param pageReqVO 分页查询
     * @return 订单结算信息表，记录店铺的结算信息分页
     */
    PageResult<ShopSettlementRespVO> getShopSettlementPage(ShopSettlementPageReqVO pageReqVO);

    /**
     * 获得订单结算信息表，记录店铺的结算信息分页
     *
     * @param pageReqVO 分页查询
     * @return 订单结算信息表，记录店铺的结算信息分页
     */
    List<TransferTemplateRespVO> excelShopSettlementPage(ShopSettlementPageReqVO pageReqVO);

    /**
     * 结算已确权订单（也是按照每个店铺）
     */
    void settlementOrder();


    /**
     * 根据平台id查询
     * @param id ptID
     * @return 店铺结算信息
     */
    List<ShopSettlementDO> selectListByPtId(Long id);
}